==> ubo_app-0.7.9-py3-none-any.whl: ubo_app/services/0-notifications/reducer.py <==
# ruff: noqa: D100, D101, D102, D103, D104, D107, N999
from __future__ import annotations

from dataclasses import replace

from redux import (
    BaseEvent,
    CompleteReducerResult,
    InitAction,
    InitializationActionError,
    ReducerResult,
)

from ubo_app.store.notifications import (
    NotificationDisplayType,
    NotificationsAction,
    NotificationsAddAction,
    NotificationsClearAction,
    NotificationsClearAllAction,
    NotificationsDisplayEvent,
    NotificationsState,
)

Action = InitAction | NotificationsAction


def reducer(
    state: NotificationsState | None,
    action: Action,
) -> ReducerResult[NotificationsState, Action, BaseEvent]:
    if state is None:
        if isinstance(action, InitAction):
            return NotificationsState(
                notifications=[],
                unread_count=0,
            )
        raise InitializationActionError(action)

    if isinstance(action, NotificationsAddAction):
        events = []
        if action.notification.display_type in (
            NotificationDisplayType.FLASH,
            NotificationDisplayType.STICKY,
        ):
            events.append(NotificationsDisplayEvent(notification=action.notification))
        if action.notification in state.notifications:
            return CompleteReducerResult(state=state, events=events)
        return CompleteReducerResult(
            state=replace(
                state,
                notifications=[*state.notifications, action.notification],
                unread_count=state.unread_count + 1,
            ),
            events=events,
        )
    if isinstance(action, NotificationsClearAction):
        return replace(
            state,
            notifications=[
                notification
                for notification in state.notifications
                if notification is not action.notification
            ],
            unread_count=state.unread_count - 1
            if action.notification in state.notifications
            else state.unread_count,
        )
    if isinstance(action, NotificationsClearAllAction):
        return replace(state, notifications=[], unread_count=0)
    return state

==> ubo_app-0.7.10-py3-none-any.whl: ubo_app/services/0-notifications/reducer.py <==
# ruff: noqa: D100, D101, D102, D103, D104, D107, N999
from __future__ import annotations

from dataclasses import replace

from kivy.utils import get_color_from_hex
from redux import (
    BaseEvent,
    CompleteReducerResult,
    InitAction,
    InitializationActionError,
    ReducerResult,
)

from ubo_app.store.led_ring import LedRingBlinkAction
from ubo_app.store.notifications import (
    Importance,
    NotificationDisplayType,
    NotificationsAction,
    NotificationsAddAction,
    NotificationsClearAction,
    NotificationsClearAllAction,
    NotificationsDisplayEvent,
    NotificationsState,
)

Action = InitAction | NotificationsAction


def reducer(
    state: NotificationsState | None,
    action: Action,
) -> ReducerResult[NotificationsState, LedRingBlinkAction, BaseEvent]:
    if state is None:
        if isinstance(action, InitAction):
            return NotificationsState(
                notifications=[],
                unread_count=0,
            )
        raise InitializationActionError(action)

    if isinstance(action, NotificationsAddAction):
        events = []
        if action.notification.display_type in (
            NotificationDisplayType.FLASH,
            NotificationDisplayType.STICKY,
        ):
            events.append(NotificationsDisplayEvent(notification=action.notification))
        if action.notification in state.notifications:
            return CompleteReducerResult(state=state, events=events)
        kivy_color = get_color_from_hex(action.notification.color)
        return CompleteReducerResult(
            state=replace(
                state,
                notifications=[*state.notifications, action.notification],
                unread_count=state.unread_count + 1,
            ),
            actions=[
                LedRingBlinkAction(
                    color=(
                        round(kivy_color[0] * 256),
                        round(kivy_color[1] * 256),
                        round(kivy_color[2] * 256),
                    ),
                    repetitions={
                        Importance.LOW: 1,
                        Importance.MEDIUM: 2,
                        Importance.HIGH: 3,
                        Importance.CRITICAL: 4,
                    }[action.notification.importance],
                ),
            ],
            events=events,
        )
    if isinstance(action, NotificationsClearAction):
        return replace(
            state,
            notifications=[
                notification
                for notification in state.notifications
                if notification is not action.notification
            ],
            unread_count=state.unread_count - 1
            if action.notification in state.notifications
            else state.unread_count,
        )
    if isinstance(action, NotificationsClearAllAction):
        return replace(state, notifications=[], unread_count=0)
    return state
